UNIT LeCroy;
{$L+}
{Utilities to read waveforms from a LeCroy 9314 digital oscilloscope
 using its GPIB interface and a CEC GPIB card on the PC side}
{20.09.94: added flag for 9310 scope}


INTERFACE


TYPE
  LeCroy_tChannels=(TA,TB,TC,TD,C1,C2,C3,C4,M1,M2,M3,M4);
  LeCroy_tType=(LC9310,LC9314);
  LeCroy_tDChannels=TA..C4;
  LeCroy_tParams=RECORD
                   Comments:ARRAY[LeCroy_tdChannels] OF STRING[80];
                   FileHeader:ARRAY[0..7] OF STRING[80];
                 END;

VAR
  LeCroy_Adr:INTEGER;
  LeCroy_Type:LeCroy_tType;
  LeCroy_Params:LeCroy_tParams;
  trname:array[LeCroy_tChannels]of string[2];{Holds the names of all channels}


PROCEDURE LeCroy_Init(InitGPIB:BOOLEAN;OscGPIBAdr:INTEGER;LCType:STRING);

{Initialize the unit. OscAdr is the oscilloscope's GPID address.
 When InitGPIB is true, the bus is initialized, too.
 LCType can be '9310' (two-channel model) or '9314' (four-channel model).
 After initialization, ControlScreen and PromptComments are false.
 At startup time, the unit is initialized without initializing the bus and
 the OscAdr is set to 4}


PROCEDURE LeCroy_Setup(ControlScreen,PromptComments:BOOLEAN);

{ ControlScreen: Tell the routines to use the screen for error messages etc.
  Set this variable to FALSE if, for example, the screen is in Graph mode
  while the LeCroy procedures are called.
 PromptComments: When TRUE, the procedure SaveFromScreen asks for a comment
  for each waveform it saves. NB.: To use this feature, set ControlScreen to
  TRUE}


PROCEDURE LeCroy_Address(OscGPIBAdr:INTEGER);

{To change the scope's GPIB address. Default Adr is 4}

PROCEDURE GetWaveform(Ch:LeCroy_tChannels;VAR Data:ARRAY OF REAL;
                      MaxPts:INTEGER;VAR nPts:INTEGER;VAR dt:REAL;
                      VAR Error:INTEGER);
{Returns the curve of channel Ch in the Real array Data, which must have
 at least MaxiPoints elements. On the LeCroy, a curve (or 'waveform') may
 have as many as 10000 points. If MaxiPts<(number of points in the curve),
 the data is 'sparsed', which means that only every nth point is transferred
 to the data array, where n=(nb of points in the curve) div MaxiPts.
 Data is in Volts. dt is the spacing between points in the Data array
 in units of seconds. nPts is the number of points the Data array actually
 contains after the transfer. Error=0 means successful transfer.}

PROCEDURE SaveFromScreen(MaxPts:INTEGER;VAR TFile:TEXT;VAR Error:INTEGER);

{Save all curves currently displayed on the oscilloscope screen in the file
 TFile, which must be open. If PromptComments and ControlScreen are both TRUE,
 the procedure asks for a comment for each curve. If a curve has more than
 MaxPts points, it is sparsed, that is, only every sth point is saved, where
 s is the largest integer so that (total # of pts in the curve)/s <= MaxPts.
}



IMPLEMENTATION


USES IEEE,Crt;

const
  Ver='0.13 (20.09.94)';
  MaxMaxPoints=2501;
  MaxChan=4;

TYPE
  tData=ARRAY[1..MaxChan,0..MaxMaxPoints+1] of INTEGER;
  tPData=^tData;
  tChDisp=ARRAY[1..MaxChan] OF LeCroy_tChannels;
  tWI=RECORD
        IData:tPData;
        nChan:INTEGER;
        nPoints:INTEGER;
        ChDisp:tChDisp;
        sparsf,vgain,voffset,trigtime,hint,hoffset,wavesource,npts:
          ARRAY[1..MaxChan]OF STRING;
        rvgain,rvoffset,rhint,rhoffset,rsparse:array[1..MaxChan] of REAL;
      END;

VAR
  OscAdr:INTEGER;
  CtrlScreen,PromptComts:BOOLEAN;

PROCEDURE writewarning(warning:STRING;Prompt:STRING);
TYPE
  String78=STRING[78];
VAR
  i,j,l:integer;
  WLine:ARRAY[1..8] OF String78;
BEGIN
  {Separate warning into lines}
  l:=1;
  j:=0;
  FOR i:=1 TO Length(Warning) DO BEGIN
    IF Warning[i]<>'|' THEN BEGIN
      j:=j+1;
      WLine[l,j]:=Warning[i];
    END ELSE BEGIN
      WLine[l,0]:=Chr(j);
      l:=l+1;
      j:=0;
    END;
  END;
  WLine[l,0]:=Chr(j);
  Window(2,10,79,l+15);
  textcolor(lightblue);
  textbackground(blue);
  clrscr;
  write(' Attention ! Ŀ');
  for i:=2 to 5+l do begin
    gotoxy(1,i);write('');
    gotoxy(78,i);write('');
  end;
  gotoxy(1,6+l);
  write('');
  FOR i:=1 TO l DO BEGIN
    gotoxy(40-(Length(WLine[i]) DIV 2),2+i);
    write(WLine[i]);
  END;
  gotoxy(40-(Length(Prompt) DIV 2),4+l);
  write(Prompt);
  window(1,1,80,25);
END;


PROCEDURE LeCroy_Init(InitGPIB:BOOLEAN;OscGPIBAdr:INTEGER;LCType:STRING);
var
  gpibstat:integer;
  i:integer;
BEGIN
  {Initialize parameters:}
  CtrlScreen:=FALSE;
  PromptComts:=FALSE;
  IF InitGPIB THEN Initialize(0,0);
  LeCroy_Adr:=OscGPIBAdr;
  IF LCType='9310' THEN LeCroy_Type:=LC9310 ELSE LeCroy_Type:=LC9314;
  {Initialisation de l' oscilloscope}
  Transmit('DCL',gpibstat);
  if GPIBStat <> 0 then begin
    WriteWarning(chr(7)+'Le LECROY ne rpond pas (Problme de cble GPIB ?)',
    'Appuyez sur <Entre> pour continuer');
    readln;
  end else begin
    for i:=0 to 32000 do;
    {Tell the scope what data format to use:}
    send(LeCroy_Adr,'COMM_ORDER LO',gpibstat);
    send(LeCroy_Adr,'COMM_FORMAT IND0,WORD,BIN',gpibstat);
    send(LeCroy_Adr,'COMM_HEADER OFF',gpibstat);
    Transmit('GTL',GPIBStat);
    {Noms des voies}
    trname[C1]:='C1';
    trname[C2]:='C2';
    trname[C3]:='C3';
    trname[C4]:='C4';
    trname[TA]:='TA';
    trname[TB]:='TB';
    trname[TC]:='TC';
    trname[TD]:='TD';
    trname[M1]:='M1';
    trname[M2]:='M2';
    trname[M3]:='M3';
    trname[M4]:='M4';
  end;
END;

PROCEDURE LeCroy_Setup;{(ControlScreen,PromptComments:BOOLEAN)}
BEGIN
  CtrlScreen:=ControlScreen;
  PromptComts:=PromptComments;
END;

PROCEDURE LeCroy_Address;{(OscGPIBAdr:INTEGER)}
BEGIN
  LeCroy_Adr:=OscGPIBAdr;
END;

function rconvert(var st:string):real;
var i:integer;
  st2:string;
  valu:real;
begin
  i:=length(st);
  while st[i]=' ' do i:=i-1;
  st2:=copy(st,23,i-22);
  val(st2,valu,i);
  rconvert:=valu;
end;

PROCEDURE TestChannels(VAR WI:tWI;VAR Error:INTEGER);
VAR
  TestChan:LeCroy_tChannels;
  LastChan:LeCroy_tChannels;
  ComStr:STRING;
  l:word;
BEGIN
  IF LeCroy_Type=LC9310 THEN LastChan:=C2 ELSE LastChan:=C4;
  WITH WI DO BEGIN
    nChan:=0;
    FOR TestChan:=TA TO LastChan do begin
      Send(LeCroy_Adr,trname[testchan]+':TRACE? ',Error);
      Enter(comstr,40,l,LeCroy_Adr,Error);
      if comstr='ON' then begin
        nchan:=nchan+1;
        chdisp[nchan]:=testchan;
      END;
    END;
  END;
END;

PROCEDURE Inquire(VAR WI:tWI;MaxPts:INTEGER;VAR Error:INTEGER);
VAR
  gpibstat:integer;
  i,j,err:integer;
  l:word;
  Chan:LeCroy_tChannels;
  ichan:integer;
  comstr:string;
  wacount:array[1..maxchan] of integer;
BEGIN
  WITH WI DO BEGIN
    Send(LeCroy_Adr,'WFSU SP,0,NP,10000',gpibstat);
    for i:=1 TO nChan DO BEGIN
      Chan:=ChDisp[i];
      {Find out appropriate sparsing factor to reduce the number of points}
      {to at most maxpts points}
      send(LeCroy_Adr,trname[Chan]+':INSP? ''WAVE_ARRAY_COUNT''',gpibstat);
      enter(comstr,40,l,LeCroy_Adr,gpibstat);
      comstr:=copy(comstr,22,11);
      while comstr[length(comstr)]=' ' do
        comstr[0]:=chr(ord(comstr[0])-1);
      val(comstr,j,err);
      wacount[i]:=j;
      j:=j div maxpts;
      if j=0 then j:=1;
      rsparse[i]:=j;
      str(j,sparsf[i]);
      str(wacount[i] div j,npts[i]);
    end;
{Vrifier que le nombre de points est le mme pour toutes les courbes}
    j:=2;
    while (j<=nchan) and (error=0) do BEGIN
      IF wacount[j]<>wacount[1] THEN Error:=1;
      j:=j+1;
    END;
    if (Error=1) AND CtrlScreen then begin
      WriteWarning(chr(7)+chr(7)+'Transfert du LECROY : Erreur d''acquisition.|'+
      'Essayer de lire les courbes une par une.','Appuyer sur <Entre> pour continuer');
    end;
  END;
END;

PROCEDURE Acquire(VAR WI:tWI;VAR Error:INTEGER);
VAR
  ichan:INTEGER;
  chname:STRING;
  GPIBStat:INTEGER;
  ComStr,Dummy:STRING;
  l:WORD;

BEGIN
  WITH WI DO BEGIN
    if CtrlScreen AND PromptComts then
      for ichan:=1 to nchan do begin
        writeln('Commentaire pour ',trname[chdisp[ichan]],' ?');
        readln(LeCroy_Params.Comments[chdisp[ichan]]);
      end;
    fillchar(IData^,sizeof(IData^),chr(0));
    for ichan:=1 to nchan do begin
{Get relevant information from WaveDesc block:}
      chname:=trname[chdisp[ichan]];
      send(LeCroy_Adr,chname+':INSP? ''VERTICAL_GAIN''',gpibstat);
      enter(vgain[ichan],40,l,LeCroy_Adr,gpibstat);
      send(LeCroy_Adr,chname+':INSP? ''VERTICAL_OFFSET''',gpibstat);
      enter(voffset[ichan],40,l,LeCroy_Adr,gpibstat);
      send(LeCroy_Adr,chname+':INSP? ''TRIGGER_TIME''',gpibstat);
      enter(trigtime[ichan],80,l,LeCroy_Adr,gpibstat);
      send(LeCroy_Adr,chname+':INSP? ''HORIZ_INTERVAL''',gpibstat);
      enter(hint[ichan],40,l,LeCroy_Adr,gpibstat);
      send(LeCroy_Adr,chname+':INSP? ''HORIZ_OFFSET''',gpibstat);
      enter(hoffset[ichan],40,l,LeCroy_Adr,gpibstat);
      send(LeCroy_Adr,chname+':INSP? ''WAVE_SOURCE''',gpibstat);
      enter(wavesource[ichan],40,l,LeCroy_Adr,gpibstat);
{Transform the strings to reals for later use}
      dummy:=vgain[ichan];
      rvgain[ichan]:=rconvert(dummy);
      dummy:=voffset[ichan];
      rvoffset[ichan]:=rconvert(dummy);
      dummy:=hint[ichan];
      rhint[ichan]:=rconvert(dummy);
      dummy:=hoffset[ichan];
      rhoffset[ichan]:=rconvert(dummy);
{Get the waveform itself:}
      comstr:='WFSU SP,'+sparsf[ichan]+',NP,'+npts[ichan];
      send(LeCroy_Adr,comstr,gpibstat);
      send(LeCroy_Adr,chname+':WF? DAT1',gpibstat);
      {7 premiers caractres de la rponse:}
      enter(ComStr,7,l,LeCroy_Adr,gpibstat);
      {lecture des autres donnes (i e forme de la courbe)}
      str(LeCroy_Adr,ComStr);
      transmit('MLA TALK '+ComStr,gpibstat);
      rarray(IData^[ichan],2*maxmaxpoints+1,l,gpibstat);
      npoints:=l div 2;
      if (gpibstat<>32) then begin
        Error:=GPIBStat+100;
        IF CtrlScreen THEN BEGIN
          writeln(chr(7),'Erreur de GPIB pendant le transfert');
          writeln('Statut du GPIB : ',gpibstat);
          readln;
        END;
      end;
    end;
  END;
 END;

procedure SaveEP(VAR WI:tWI;VAR TFile:TEXT);
VAR
  IChan,i:INTEGER;
begin
  WITH WI DO BEGIN
    writeln(TFile,';Transfert du LECROY (version ',ver,')');
    writeln(TFile,';/aa 1 0 "',copy(trigtime[1],23,40),'" A');
    writeln(TFile,'/et x "t [s]"');
    writeln(TFile,'/et y "signal [V]"');
    for ichan:=nchan downto 1 do begin
      writeln(TFile,'/sa l "',LeCroy_Params.Comments[chdisp[ichan]],'" ',ichan+1);
    end;
    for i:=0 to npoints do begin
      write(TFile,rhoffset[1]+i*rsparse[1]*rhint[1]:14);
        for ichan:=1 to nchan do begin
        write(TFile,' ',rvgain[ichan]*IData^[ichan,i]-rvoffset[ichan]:13);
      end;
      writeln(TFile);
    end;
  END;
end;

PROCEDURE SaveFromScreen(MaxPts:INTEGER;VAR TFile:TEXT;VAR Error:INTEGER);
LABEL 99;
VAR
  WI:tWI;
BEGIN
  Error:=0;
  New(WI.IData);
  TestChannels(WI,Error);
  IF Error<>0 THEN GOTO 99;
  Inquire(WI,MaxPts,Error);
  IF Error<>0 THEN GOTO 99;
  Acquire(WI,Error);
  IF Error<>0 THEN GOTO 99;
  SaveEP(WI,TFile);
99:
  Dispose(WI.IData);
END;

PROCEDURE GetWaveform(Ch:LeCroy_tChannels;VAR Data:ARRAY OF REAL;
                      MaxPts:INTEGER;VAR nPts:INTEGER;VAR dt:REAL;
                      VAR Error:INTEGER);
LABEL 99;
VAR
  WI:tWI;
  i:INTEGER;
  Gain,Offset:REAL;
BEGIN
  Error:=0;
  New(WI.IData);
  WI.nChan:=1;
  WI.ChDisp[1]:=Ch;
  Inquire(WI,MaxPts,Error);
  IF Error<>0 THEN GOTO 99;
  Acquire(WI,Error);
  IF Error<>0 THEN GOTO 99;
  nPts:=WI.nPoints;
  dt:=WI.rsparse[1]*WI.rhint[1];
  Gain:=WI.rvgain[1];
  Offset:=WI.rvoffset[1];
  FOR i:=0 TO WI.nPoints DO
    Data[i]:=Gain*WI.IData^[1,i]-Offset;
99:
  Dispose(WI.IData);
END;

BEGIN
  LeCroy_Init(TRUE,4,'9310');
END.
