unit Timing;

{
  This Unit implements a good delay routine using the DAS 1601 Counter 0.
  The computer is synchronized to the counter to increase the reliability.
  startCounter has to be called shortly (less than the first wait-time and
  less than half a second) before the first Wait command.
}

interface

var DAS1601Present:boolean;
procedure startCounter;
procedure TimingWait(time:real); {time in ms}
procedure SimpleWait(time:real); {time in ms}

implementation
uses crt;

CONST PCCalibration=1193.18;

CONST
  {internal timer registers}
  PCtimer0=$40;
  PCtimer1=$41;
  PCtimer2=$42;
  PCtimerctl=$43;
  PCtimergate=$61;
  {DAS-1601 registers:}
  adr=$310;
  ad0=adr+$0;
  ad1=adr+$1;
  mux=adr+$2;
  dio=adr+$3;
  da0lb=adr+$4;
  da0hb=adr+$5;
  da1lb=adr+$6;
  da1hb=adr+$7;
  statusa=adr+$8;
  ctrl=adr+$9;
  cnten=adr+$A;
  gain=adr+$B;
  DAS1601timer0=adr+$C;
  DAS1601timer1=adr+$D;
  DAS1601timer2=adr+$E;
  DAS1601timerctl=adr+$F;
  piopa=adr+$400;
  piopb=adr+$401;
  piopc=adr+$402;
  pioctl=adr+$403;
  disarm=adr+$404;
  bmden=adr+$405;
  mden1600=adr+$406;
  statusb=adr+$407;

CONST
  CNT0=0;
  CNT1=64;
  CNT2=128;
  Readback=128+64;

  CNTLatch=0;
  LowByte=16;
  HighByte=32;
  BothBytes=16+32;

  PulseOnTerminalCount=0;
  ProgrammableOneShot=2;
  RateGenerator=4;
  SquareWaveGenerator=2+4;
  SoftwareTriggeredStrobe=8;
  HardwareTriggeredStrobe=8+2;

  Binary=0;
  BCD=1;

  NoCNTLatch=32;
  DontReadStatus=16;
  ReadStatus=0;
  C0=2;
  C1=4;
  C2=8;

  ExternalClock=0;
  InternalClock=2;

  OldCounter:word=$ffff;

var C:byte;
    CounterNR:byte;
    timerctl:word;
    timer:word;
    counts_ms:double;
    GoalDistance:word;
var OldTimerInt,oldexitproc:pointer;
    OldNow,Zaehler:longint;


procedure StartCounter;
begin
  if DAS1601Present then begin
    port[cnten]:=InternalClock;       {enable counter 0}
  end else begin
    Zaehler:=0;
    Port[PCtimerctl]:=ReadBack or DontReadStatus or CNTLatch or C0;
    OldNow:=(Port[PCtimer0]*256+Port[PCtimer0]) div 2 + Zaehler*32768;
  end;
  Port[timerctl]:=CounterNR or BothBytes or PulseOnTerminalCount or Binary;
  Port[timer]:=$FF;
  Port[timer]:=$FF;    {counter 0 counting down form 65535 with 100kHz}
  OldCounter:=$FFFF;
end;

procedure TimingWait(time:real); {time in ms}
var goal,now,help,n,highcount:word;
    GoalNow,cnt:longint;
begin
  if DAS1601Present then begin
    if (time<60000/counts_ms) then cnt:=round(time*counts_ms) else begin
      if (time>1000000) then begin
        writeln('Timing : Wait : You surely don''t want to wait that long');
        time:=10000;
      end;
      highcount:=trunc(time/10);
      cnt:=round((time-highcount*10)*counts_ms);
      if (cnt<2*GoalDistance) then repeat
        cnt:=cnt+round(counts_ms*10);
        if highcount>0 then highcount:=highcount-1
        else writeln('Timing : Wait : GoalDistance error');
      until (cnt>=2*GoalDistance);
      for n:=1 to highcount do TimingWait(10);
    end;
    if (cnt<GoalDistance/3) then writeln('Timing : Wait : delay too short');
    {$R-} goal:=Oldcounter-cnt;
    if (Oldcounter<cnt) then begin
      {the counter has to jump from 0 to $FFFF}
      if (65535-goal>GoalDistance) then begin
        {the goal is not to near to the boarder, so we can wait until he jumped}
        repeat
          Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C; {read counter}
          now:=Port[timer]*256+Port[timer];
        until (now>Oldcounter);
      end else begin
        {the goal is very near the boarder, so we wait until the counter is near the boarder
         or jumped over it}
        repeat
          Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
          now:=Port[timer]*256+Port[timer];
        until (now<GoalDistance) or (now>65535-GoalDistance);
        {we do not want to be disturbed, so switch interrupts off}
        asm cli end;
        {wait until goal is reached}
        repeat
          Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
          now:=Port[timer]*256+Port[timer];
        until (now>65000) and (now<=goal);
        asm sti end;  {switch interrupts on again}
        Oldcounter:=goal;  {store last goal}
        exit;         {we have reached the goal}
      end;
    end;
    Oldcounter:=goal; {store last goal}
    {wait until the counter is near the goal}
    repeat
      Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
      now:=Port[timer]*256+Port[timer];
    until (now-goal<GoalDistance);
    asm cli end; {switch interrupts off}
    {wait until the counter reaches the goal}
    repeat
      Port[timerctl]:=ReadBack or DontReadStatus or CNTLatch or C;
      now:=Port[timer]*256+Port[timer];
    until (now<=goal);
    asm sti end; {switch interrupts on}
  end else begin
    Goal:=
    Port[PCtimerctl]:=ReadBack or DontReadStatus or CNTLatch or C0;
    now:=(Port[PCtimer0]*256+Port[PCtimer0]) div 2 + Zaehler*32768;

  end;
end;

procedure SimpleWait(time:real); {time in ms}
var cnt:word;
    status:byte;
begin
  if DAS1601Present then begin
    port[cnten]:=InternalClock;       {enable counter 0}
  end else begin
    port[PCtimergate]:=3;
  end;
  cnt:=round(time*counts_ms); {program counter}
  Port[timerctl]:=CounterNR or BothBytes or PulseOnTerminalCount or Binary;
  Port[timer]:=Lo(cnt);
  Port[timer]:=Hi(cnt);
  repeat                {wait until he reached 0}
    Port[timerctl]:=ReadBack or NoCNTLatch or ReadStatus or C;
    status:=Port[timer];
  until (status and 128)>0;
end;

function checkDAS1601:boolean;
begin
  checkDAS1601:=DAS1601present;
end;

procedure InitTiming;
var n,now,oldnow,cnt:word;
begin
  port[cnten]:=InternalClock;       {enable counter 0}
  Port[DAS1601timerctl]:=CNT0 or BothBytes or PulseOnTerminalCount or Binary;
  cnt:=round(5*100); {program counter 5ms}
  Port[DAS1601timer0]:=Lo(cnt);
  Port[DAS1601timer0]:=Hi(cnt);
  Port[DAS1601timerctl]:=ReadBack or DontReadStatus or CNTLatch or C0;
  oldnow:=Port[DAS1601timer0]*256+Port[DAS1601timer0];
  n:=0;
  repeat
    delay(1);
    Port[DAS1601timerctl]:=ReadBack or DontReadStatus or CNTLatch or C0;
    now:=Port[DAS1601timer0]*256+Port[DAS1601timer0];
    n:=n+1;
  until (now<oldnow) or (n>10);
  DAS1601present:=(n<10) or (now<oldnow);

  {DAS1601present:=false;}

  if DAS1601present then begin
    GoalDistance:=10;
    timerctl:=DAS1601timerctl;
    timer:=DAS1601timer0;
    CounterNR:=CNT0;
    C:=C0;
    counts_ms:=100;
  end else begin
    GoalDistance:=200;
    timerctl:=PCtimerctl;
    timer:=PCtimer2;
    CounterNR:=CNT2;
    C:=C2;
    counts_ms:=PCCalibration;
    Zaehler:=0;
    getIntvec(8,OldTimerInt);
    asm CLI end;
      SetIntVec(8,@NewTimer);
    asm STI end;
  end;
end;

Procedure NewTimer;Interrupt;
var R:registers;
begin
  Inc(Zaehler);
  asm
    PUSHF
    CALL OldTimerInt
  end;
end;

procedure myexitproc;far;
begin
  if not DAS1601present then begin
    asm CLI end;
      SetIntVec(8,OldTimerInt);
    asm STI end;
  end;
  exitproc:=oldexitproc;
end;

begin
  InitTiming;
  oldexitproc:=exitproc;
  exitproc:=@myexitproc;
end.